source("common.R")

unit.root.test <- function(comm)
{
  x <- get.log.monthly.data(comm)
  pp <- pp.test(x, type = "Z(t_alpha)", lshort = FALSE)
  kpss <- kpss.test(x, null = "Trend", lshort = FALSE)

  if (pp$p.value == kpss$p.value)
    stat = NA
  else if (kpss$p.value == 0.1 & pp$p.value > 0.1)
    stat = NA
  else
    stat = (kpss$p.value > pp$p.value)

  list(stat=stat, pp=pp, kpss=kpss)  
}

## Estime un modèle ARIMA, par maximum de vraisemblance et en sélectionnant le meilleur AIC
## Possibilité de continuer les calculs à partir d'une estimation inachevée passée dans old.fit
fit.arima <- function(data, diff = FALSE, old.fit = NULL, order.max = 17, order.max.seasonal = 2)
{
  if (any(is.na(data)))
    stop("NAs in data")

  if (!is.null(old.fit))
    {
      if (!inherits(old.fit, "fit"))
        stop("old.fit must be of class fit.")

      best.arima <- old.fit$arima
      best.bp <- old.fit$bp
      done <- old.fit$done
    }
  else
    {
      best.arima <- NA
      done <- list()
    }

  n <- length(data)

  xreg <- time(data) - (ref.year + (ref.month - 1) / 12)

  for (order in 0:order.max)
    for (p in 0:order)
      for (P in 0:order.max.seasonal)
        for (Q in 0:(order.max.seasonal - P))
          for (d in 0:diff)
            {
              q <- order - p
              D <- diff - d

              if (any(unlist(lapply(done, function(x) { all(x == c(p,d,q,P,D,Q)) }))))
                next

              cat("Order", p, d, q, "-", P, D, Q, ": ")

              arima <- tryCatch(arima0(data, order = c(p, d, q), seasonal = c(P, D, Q), xreg = xreg, method = "ML"),
                                interrupt = function(e) { "interrupt" },
                                error = function(e) { "error" })

              if (identical(arima, "interrupt"))
                return(list(arima = best.arima, bp = best.bp, done = done))
              else if (!identical(arima, "error"))
                {
                  if (arima$code != 0)
                    cat("Convergence problem", fill = TRUE)
                  else if (min(diag(arima$var.coef)) <= 0)
                    cat("Negative variance", fill = TRUE)
                  else
                    {
                      cat("AIC", arima$aic)

                      bp <- Box.test(arima$residuals, lag = 24)

                      if (bp$p.value < 0.05)
                        cat(" Fails Box-Pierce test")
                      else if (is.na(best.arima) || arima$aic < best.arima$aic)
                        {
                          best.arima <- arima
                          best.bp <- bp
                          cat(" BEST !!")
                        }

                      cat(fill = TRUE)
                    }
                }

              done <- c(done, list(c(p,d,q,P,D,Q)))
            }
  
  list(arima = best.arima, bp = best.bp, done = done)
}

## Estimation complète d'un modèle pour une commodité
## Les arguments supplémentaires sont passés à fit.arima()
## Renvoie un objet de classe "fit"
fit.model <- function(comm, ...)
{
  ur <- unit.root.test(comm)
  data <- get.log.monthly.data(comm)

  if (is.na(ur$stat))
    {
      return(NA)
    }
  else if (ur$stat)
    {
      arima <- fit.arima(data, diff = FALSE, ...)
    }
  else
    {
      ref.val <- exp(as.numeric(window(data, start=c(ref.year, ref.month), end=c(ref.year, ref.month))))
      arima <- c(list(ref.val = ref.val), fit.arima(data, diff = TRUE, ...))
    }

  fit <- c(list(comm = comm, start = start(data), end = end(data), nobs = length(data)), arima, ur)

  structure(fit, class = "fit")
}

print.fit <- function(f, ...)
{
  cat("Commodity:", f$comm, fill = TRUE)
  cat("Number of observations:", f$nobs, fill = TRUE)
  cat("Dates: ", f$start[2], "/", f$start[1], "-", f$end[2], "/", f$end[1], sep="", fill = TRUE)
  cat("Stationary:", f$stat, fill = TRUE)
  cat(fill = TRUE)
  cat("Arima fit:")
  print(f$arima)
  cat(fill = TRUE)
  cat("Box-Pierce test:", round(f$bp$statistic, 1), "(", round(f$bp$p.value, 3), ")", fill = TRUE)
}

## Construit la table de l'annexe B de Villemot (2004)
print.arima.table <- function(fits, limits = list(ar = 13, ma = 12, sar = 1, sma = 1))
{
  for (type in c("ar", "sar", "ma", "sma"))
    {
      for (i in 1:limits[[type]])
        {
          name <- paste(type, i, sep = "")
          cat(name)
          for (comm in names(comm.names))
            {
              cat(" & ")
              onx <- fits[[comm]]$arima$coef[name]
              if (!is.na(onx))
                cat(formatC(onx, digits = 3, format = "f"))
            }
          cat("\\\\", fill = TRUE)

          for (comm in names(comm.names))
            {
              cat(" & ")
              try(cat("(", formatC(sqrt(fits[[comm]]$arima$var.coef[name, name]), digits = 3, format = "f"), ")", sep = ""), silent = TRUE)
            }
          cat("\\\\", fill = TRUE)
        }
      cat("\\hline", fill = TRUE)
    }
}


## Run estimations

fits <- list()
for (comm in names(volumes))
  {
    print(comm)
    fits[[comm]] <- fit.model(comm, old.fit = fits[[comm]], order.max = 21, order.max.seasonal = 0)
    fits[[comm]] <- fit.model(comm, old.fit = fits[[comm]], order.max = 15, order.max.seasonal = 1)
    fits[[comm]] <- fit.model(comm, old.fit = fits[[comm]], order.max = 13, order.max.seasonal = 2)
    fits[[comm]] <- fit.model(comm, old.fit = fits[[comm]], order.max = 6, order.max.seasonal = 3)
    print.fit(fits[[comm]])
  }
save(fits, file="data/fits.RData")
