source("common.R")

data(fits)

## Crée une série temporelle mensuelle simulée sur un horizon de "h.months" moi à partir de ref.year/ref.month

sim.fit <- function(fit, h.months, ...)
{
  if (!inherits(fit, "fit"))
    stop("This is not a fit class instance.")

  ## Prend en argument des polyonomes (coef et coefS) sans leur coefficient constant supposé égal à 1
  ## o et oS sont les ordres respectifs des polynômes
  ## Renvoie le résultat de la multiplication de coef(X) par coefS(X^12), sans le coefficient constant
  poly.mult <- function(coef, o, coefS, oS)
  {
    r <- numeric(1 + o + oS * 12)
    coef <- c(1, coef)
    coefS <- c(1, coefS)

    for(i in 0:o)
      for(j in 0:oS)
        r[1 + i + 12 * j] <- r[1 + i + 12 * j] + coef[1 + i] * coefS[1 + j]

    r[-1]
  }


  arima <- fit$arima

  p <- arima$arma["ar"]
  q <- arima$arma["ma"]
  d <- arima$arma["diff"]

  P <- arima$arma["sar"]
  Q <- arima$arma["sma"]
  D <- arima$arma["sdiff"]

  if (D > 0)
    stop("D > 0 not supported.")

  model.ar <-  if (p > 0) arima$coef[paste("ar", 1:p, sep = "")] else numeric()
  model.ma <-  if (q > 0) arima$coef[paste("ma", 1:q, sep = "")] else numeric()
  model.sar <-  if (P > 0) arima$coef[paste("sar", 1:P, sep = "")] else numeric()
  model.sma <-  if (Q > 0) arima$coef[paste("sma", 1:Q, sep = "")] else numeric()

  if (P == 0)
    model.ar.final <- model.ar
  else
    model.ar.final <- - poly.mult(-model.ar, p, -model.sar, P)

  if (Q == 0)
    model.ma.final <- model.ma
  else
    model.ma.final <- poly.mult(model.ma, q, model.sma, Q)

  order = c(p + 12 * P, d, q + 12 * Q)

  x <- arima.sim(list(order = order, ar = model.ar.final, ma = model.ma.final), h.months - d, sd = sqrt(arima$sigma2), ...)

 if (fit$stat)
   {
     mean <- as.numeric(arima$coef["intercept"])
     trend <- arima$coef["xreg"]
     x <- exp(x + rep(mean, h.months) + trend * (0:(h.months-1)) / 12)
   }
 else
   {
     trend <- as.numeric(arima$coef["xreg1"])
     x <- exp(x + trend * (0:(h.months-1)) / 12) * fit$ref.val
   }

  ts(x, start=c(ref.year, ref.month), frequency = 12)
}

## Crée une liste de n.sims simulations en séries temporelles *annuelles*
make.sims <- function(fit, h.years, n.sims)
{
  sims <- list()

  for (i in 1:n.sims)
    {
      x <- sim.fit(fit, h.years * 12 + 1)
      x2 <- window(x, start=c(ref.year, ref.month), frequency = 1)
      sims[[i]] <- x2
    }

  structure(sims, class = "sims", fit = fit, h.years = h.years)
}

print.sims <- function(s, ...)
{
  cat("Simulations of", attr(s, "fit")$comm, fill = TRUE)
  cat("Horizon:", attr(s, "h.years"), "years", fill = TRUE)
  cat("Number of simulations:", length(s), fill = TRUE)
}

## Perform simulations

for (comm in names(sims))
{
  assign(sims[[comm]], make.sims(fits[[comm]], sims.horizon, sims.length))
  save(list = sims[[comm]], file = paste("data/", sims[[comm]], ".RData", sep = ""))
  rm(list = sims[[comm]])
}
